// @check-accepted: Examples n<=2000 k=R-L+1 k=1 n<=50000 tests
#include <ios>
#include <vector>
#include <numeric>
#include <algorithm>
#include <iostream>
using namespace std;
using ll = long long;

const ll mod = 1e9 + 7;

struct query{
    int l;
    int r;
    int id;
    int k;
    int bs = 0;
    vector<int> nodes;
};

ll inv(ll a){
    ll r = 1;
    for(ll e = mod - 2; e; e/=2){
        if(e % 2)r = (r * a)%mod;
        a = (a*a) % mod;
    }
    return r;
}

int main(){
    cin.tie(0);
    ios_base::sync_with_stdio(0);
    int N,Q;
    cin >> N >> Q;
    vector<long long> V(N);
    for(auto &x: V)cin >> x;

    vector<query> queries(Q);
    for(int i = 0; i < Q; ++i){
        int l, r, k;
        cin >> l >> r >> k;
        queries[i] = query{l, r+1, i, k};
    }
    
    vector<int> pos_sorted(N);
    iota(pos_sorted.begin(), pos_sorted.end(), 0);
    sort(pos_sorted.begin(), pos_sorted.end(), [&](int a, int b) -> bool{
        return tie(V[a], a) > tie(V[b], b);
    });

    vector<int> fenwick(N + 1);
    
    auto upd = [&](int pos){
        for(; pos < fenwick.size(); pos += pos&-pos)
            ++fenwick[pos];
    };

    auto cnt = [&](int pos){
        int r = 0;
        for(; pos; pos -= pos&-pos)
            r += fenwick[pos];
        return r;
    };
    
    for(int i = 1 << 20; i; i/=2){
        fenwick.assign(N + 1, 0);
        sort(queries.begin(), queries.end(), [&](auto a, auto b) -> bool {
            return a.bs < b.bs;
        });
        int last = 0;
        for(auto &q: queries){
            if(q.bs + i > N)continue;
            while(last < q.bs + i)
                upd(pos_sorted[last++] + 1);
            if(cnt(q.r) - cnt(q.l) <= q.k)q.bs += i;
        }

    }

    sort(queries.begin(), queries.end(), [&](auto a, auto b) -> bool {
        return a.bs < b.bs;
    });

    vector<ll> fwprod(N + 1, 1);
    
    auto updp = [&](int pos){
        ll v = V[pos - 1];
        for(; pos < fenwick.size(); pos += pos&-pos)
            fwprod[pos] = (fwprod[pos] * v) % mod;
    };

    auto pd = [&](int pos){
        ll r = 1;
        for(; pos; pos -= pos&-pos)
            r = (r * fwprod[pos]) % mod;
        return r;
    };

    int last = 0;
    vector<ll> ans(Q);


    for(auto &q: queries){
        while(last < q.bs)
            updp(pos_sorted[last++]+ 1);
        ans[q.id] = (pd(q.r) * inv(pd(q.l))) % mod;
    }       


    for(auto x: ans)cout << x << endl;
}
